import os
from pkg_resources import iter_entry_points

import click

from . import state

DEFAULT_CONFIGFILE = os.path.expanduser('~/.gg.json')


class Config(object):
    def __init__(self):
        self.verbose = False  # default
        self.configfile = DEFAULT_CONFIGFILE


pass_config = click.make_pass_decorator(Config, ensure=True)


@click.group()
@click.option(
    '-v', '--verbose',
    is_flag=True
)
@click.option(
    '-c', '--configfile',
    default=DEFAULT_CONFIGFILE,
    help='Path to the config file (default: {})'.format(DEFAULT_CONFIGFILE)
)
@pass_config
def cli(config, configfile, verbose):
    """A glorious command line tool to make your life with git, GitHub
    and Bugzilla much easier."""
    config.verbose = verbose
    config.configfile = configfile
    if not os.path.isfile(configfile):
        state.write(configfile, {})


# load in the builtin apps
from .builtins import bugzilla  # NOQA
from .builtins import github  # NOQA

# Simply loading all installed packages that have this entry_point
# will be enough. Each plugin automatically registers itself with the
# cli click group.
for entry_point in iter_entry_points(group='gg.plugin', name=None):
    entry_point.load()



import re
import json
import getpass
import urllib.parse

import click
import requests

from gg.utils import error_out, success_out, info_out
from gg.state import read, update, remove
from gg.main import cli, pass_config

GITHUB_URL = 'https://api.github.com'


@cli.group()
@click.option(
    '-u', '--github-url',
    default=GITHUB_URL,
    help='URL to GitHub instance (default: {})'.format(
        GITHUB_URL
    )
)
@pass_config
def github(config, github_url):
    """For setting up a GitHub API token."""
    config.github_url = github_url


@github.command()
@click.argument('token', default='')
@pass_config
def token(config, token):
    """Store and fetch a GitHub access token"""
    if not token:
        if config.verbose:
            info_out(
                'To generate a personal API token, go to:\n\n\t'
                'https://github.com/settings/tokens\n\n'
                'To read more about it, go to:\n\n\t'
                'https://help.github.com/articles/creating-an-access'
                '-token-for-command-line-use/\n\n'
                'Remember to enable "repo" in the scopes.'
            )
        token = getpass.getpass('GitHub API Token: ').strip()
    url = urllib.parse.urljoin(config.github_url, '/user')
    assert url.startswith('https://'), url
    response = requests.get(
        url,
        headers={
            'Authorization': 'token {}'.format(token)
        }
    )
    if response.status_code == 200:
        update(config.configfile, {
            'GITHUB': {
                'github_url': config.github_url,
                'token': token,
                'login': response.json()['login'],
            }
        })
        name = response.json()['name'] or response.json()['login']
        success_out('Hi! {}'.format(name))
    else:
        error_out(
            'Failed - {} ({})'.format(
                response.status_code,
                response.content
            )
        )


@github.command()
@pass_config
def burn(config):
    """Remove and forget your GitHub credentials"""
    state = read(config.configfile)
    if state.get('GITHUB'):
        remove(config.configfile, 'GITHUB')
        success_out('Forgotten')
    else:
        error_out('No stored GitHub credentials')


def get_title(config, org, repo, number):
    base_url = GITHUB_URL
    headers = {}
    state = read(config.configfile)
    credentials = state.get('GITHUB')
    if credentials:
        base_url = state['GITHUB']['github_url']
        headers['Authorization'] = 'token {}'.format(
            credentials['token']
        )
        if config.verbose:
            info_out('Using API token: {}'.format(
                credentials['token'][:10] + '…'
            ))
    url = urllib.parse.urljoin(
        base_url,
        '/repos/{}/{}/issues/{}'.format(
            org, repo, number
        )
    )
    if config.verbose:
        info_out('GitHub URL: {}'.format(url))
    assert url.startswith('https://'), url
    response = requests.get(url, headers=headers)
    if response.status_code == 200:
        return response.json()['title']
    if config.verbose:
        info_out('GitHub Response: {}'.format(response))


@github.command()
@click.option(
    '-i', '--issue-url',
    help='Optionally test fetching a specific issue'
)
@pass_config
def test(config, issue_url):
    """Test your saved GitHub API Token."""
    state = read(config.configfile)
    credentials = state.get('GITHUB')
    if not credentials:
        error_out('No credentials saved. Run: gg github token')
    if config.verbose:
        info_out('Using: {}'.format(credentials['github_url']))

    if issue_url:
        github_url_regex = re.compile(
            'https://github.com/([^/]+)/([^/]+)/issues/(\d+)'
        )
        org, repo, number = github_url_regex.search(issue_url).groups()
        title = get_title(config, org, repo, number)
        if title:
            info_out('It worked!')
            success_out(title)
        else:
            error_out('Unable to fetch')
    else:
        url = urllib.parse.urljoin(
            credentials['github_url'],
            '/user'
        )
        assert url.startswith('https://'), url
        response = requests.get(url, headers={
            'Authorization': 'token {}'.format(
                credentials['token']
            )
        })
        if response.status_code == 200:
            success_out(json.dumps(response.json(), indent=2))
        else:
            error_out('Failed to query - {} ({})'.format(
                response.status_code,
                response.json()
            ))



import datetime
import json

from .utils import get_repo_name


def read(configfile):
    with open(configfile, 'r') as f:
        return json.load(f)


def write(configfile, data):
    with open(configfile, 'w') as f:
        json.dump(data, f, indent=2, sort_keys=True)


def save(configfile, description, branch_name, **extra):
    # state = read(configfile)
    repo_name = get_repo_name()
    key = '{}:{}'.format(repo_name, branch_name)
    new = {key: extra}
    new[key].update({
        'description': description,
        'date': datetime.datetime.now().isoformat(),
    })
    update(configfile, new)


def update(configfile, data):
    state = read(configfile)
    state.update(data)
    write(configfile, state)


def remove(configfile, key):
    state = read(configfile)
    del state[key]
    write(configfile, state)



import os

import click
import git


def get_repo(here='.'):
    if os.path.abspath(here) == '/':  # hit rock bottom
        raise git.InvalidGitRepositoryError('Unable to find repo root')
    if os.path.isdir(os.path.join(here, '.git')):
        return git.Repo(here)
    return get_repo(os.path.join(here, '..'))


def get_repo_name():
    repo = get_repo()
    return os.path.basename(repo.working_dir)


def error_out(msg):
    click.echo(click.style(msg, fg='red'))
    raise click.Abort


def success_out(msg):
    click.echo(click.style(msg, fg='green'))


def info_out(msg):
    click.echo(msg)



import json
import getpass
import urllib.parse

import click
import requests

from gg.utils import error_out, success_out, info_out
from gg.state import read, update, remove
from gg.main import cli, pass_config

BUGZILLA_URL = 'https://bugzilla.mozilla.org'


@cli.group()
@click.option(
    '-u', '--bugzilla-url',
    default=BUGZILLA_URL,
    help='URL to Bugzilla instance (default: {})'.format(
        BUGZILLA_URL
    )
)
@pass_config
def bugzilla(config, bugzilla_url):
    """General tool for connecting to Bugzilla. The default URL
    is that for bugzilla.mozilla.org but you can override that.
    Once you're signed in you can use those credentials to automatically
    fetch bug summaries even of private bugs.
    """
    config.bugzilla_url = bugzilla_url


@bugzilla.command()
@pass_config
def login(config):
    """Go fetch a Bugzilla cookie"""
    state = read(config.configfile)
    default = getpass.getuser()
    if state.get('BUGZILLA'):
        if state['BUGZILLA'].get('login'):
            default = state['BUGZILLA']['login']

    login = input(
        'Username/Email [{}]: '.format(default)
    ).strip() or default
    password = getpass.getpass(
        'Password (Will never be stored!): '
    )
    if not password:
        error_out('No password :(')

    url = urllib.parse.urljoin(config.bugzilla_url, '/rest/login')
    assert url.startswith('https://'), url
    response = requests.get(url, params={
        'login': login,
        'password': password,
    })

    if response.status_code == 200:
        token = response.json()['token']
        update(config.configfile, {
            'BUGZILLA': {
                'bugzilla_url': config.bugzilla_url,
                'login': login,
                'token': token,
            }
        })
        success_out('Yay! It worked!')
    else:
        error_out('Failed - {} ({})'.format(
            response.status_code,
            response.json()
        ))


@bugzilla.command()
@pass_config
def logout(config):
    """Remove and forget your Bugzilla credentials"""
    state = read(config.configfile)
    if state.get('BUGZILLA'):
        remove(config.configfile, 'BUGZILLA')
        success_out('Forgotten')
    else:
        error_out('No stored Bugzilla credentials')


def get_summary(config, bugnumber):
    params = {
        'ids': bugnumber,
        'include_fields': 'summary'
    }
    base_url = BUGZILLA_URL

    state = read(config.configfile)

    credentials = state.get('BUGZILLA')
    if credentials:
        # cool! let's use that
        base_url = credentials['bugzilla_url']
        params['token'] = credentials['token']

    url = urllib.parse.urljoin(
        base_url,
        '/rest/bug/'
    )
    assert url.startswith('https://'), url
    response = requests.get(url, params=params)
    if response.status_code == 200:
        return response.json()['bugs'][0]['summary']


@bugzilla.command()
@click.option(
    '-b', '--bugnumber',
    type=int,
    help='Optionally test fetching a specific bug'
)
@pass_config
def test(config, bugnumber):
    """Test your saved Bugzilla credentials."""
    state = read(config.configfile)
    credentials = state.get('BUGZILLA')
    if not credentials:
        error_out('No credentials saved. Run: gg bugzilla login')
    if config.verbose:
        info_out('Using: {}'.format(credentials['bugzilla_url']))

    if bugnumber:
        summary = get_summary(config, bugnumber)
        if summary:
            success_out('It worked!')
            info_out(summary)
        else:
            error_out('Unable to fetch')
    else:
        url = urllib.parse.urljoin(
            credentials['bugzilla_url'],
            '/rest/whoami'
        )
        assert url.startswith('https://'), url
        response = requests.get(url, params={
            'token': credentials['token'],
        })
        if response.status_code == 200:
            success_out(json.dumps(response.json(), indent=2))
        else:
            error_out('Failed to query - {} ({})'.format(
                response.status_code,
                response.json()
            ))



from os import path
from setuptools import setup, find_packages

_here = path.dirname(__file__)


setup(
    name='gg',
    version='0.0.10',
    author='Peter Bengtsson',
    author_email='mail@peterbe.com',
    url='https://github.com/peterbe/gg',
    description='Git and GitHub for the productivity addicted',
    long_description=open(path.join(_here, 'README.rst')).read(),
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'click',
        'colorama',
        'requests',
        'GitPython',
    ],
    entry_points="""
        [console_scripts]
        gg=gg.main:cli
    """,
    setup_requires=['pytest-runner'],
    tests_require=['pytest', 'pytest-mock'],
)
